<?php
/**
* HELIUM V Synchronisation Modul for Magento 1.7.x
*
* Copyright (C) 2013 HELIUM V IT-Solutions GmbH
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU Affero General Public License as
* published by the Free Software Foundation, either version 3 of the
* License, or (at your option) any later version.
*
* According to sec. 7 of the GNU Affero General Public License, version 3,
* the terms of the AGPL are supplemented with the following terms:
*
* "Helium V" and "Helium 5" are registered trademarks of HELIUM V IT-Solutions GmbH.
* The licensing of the Program under the AGPL does not imply a trademark license.
* Therefore any rights, title and interest in our trademarks remain entirely with us.
* If you want to propagate modified versions of the Program under the
* name "Helium V" or "Helium 5", you may only do so if you have a written permission
* by Helium V IT-Solutions GmbH (to acquire a permission please contact
* Helium V IT-Solutions at trademark@heliumv.com).
*      
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Affero General Public License for more details.
*
* You should have received a copy of the GNU Affero General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
* 
* @category Heliumv
* @package  Heliumv_Synchronization
* @author   Maximilian Hehenwarter <developers@heliumv.com>
* @license  http://www.gnu.org/licenses/  Affero General Public License (AGPL 3.0) 
*/

/**
* In dieser Klasse werden alle Cronjob Aufrufe für Das HeliumV Modul zusammengefasst
* Der Aufruf wird in der config.xml auf die funktioni hier hinterlegt und daraus dann die eigendliche Verarbeitung aufgerufen
*/
class Heliumv_Synchronization_Model_Cronjob
{
    
    /**
    * Logfiledeklaration, zugriff mit self::$logfile_name
    */
    private static $log_file="hv_cronjobcalls.log";
    
    
    /**
    * cronjob testaufruf
    * benötigt config.xml und laufenden aufruf von localhost/magento2/cron.php
    * siehe auch http://www.magentocommerce.com/wiki/1_-_installation_and_configuration/how_to_setup_a_cron_job
    * @param $schedule
	*/
    public function test_old($schedule)
	{
    
        //aktualisiert artikel mit external_pid 2476, direktaufruf findet klasse nicht
        //Heliumv_Synchronization_AjaxController::testAction();
        
        mage::log("Cronjob gelaufen um ".gmdate("Y-m-d H:i:s"));
        
        
        //diverse tests zu artikel neu erstellen bzw aktualisieren
        
        //initialisierung
        //$store_id=Mage::app()->getStore()->getId();
        
        //schrit1: interne produktid ermitteln
        //$product_id=$this->convertExternalIDtoProductID(2476);
        $product_id="1";
        Mage::log('Test: Arbeitet mit Magento Produkt ID: '.$product_id);
        
        //schritt2: produkt mit interner ID öffnen
        $product = Mage::getModel('catalog/product');
        $product->setStoreId(Heliumv_Synchronization_Helper_Master::getStoreId());
        $product->load($product_id);
        
        //schritt3: 
        $product->setAttributeSetId(Heliumv_Synchronization_Helper_Master::getDefaultAttributeSetId('catalog/product'));     
        $product->setName("Artikelname changed um ".gmdate("Y-m-d H:i:s"));

        try {
            $product->save();                                
        } catch (Exception $ex){
            Mage::log("Test: Error bei Speichern von Artikel ID".$product_id);
            Mage::log($ex->getMessage());
        }              
        
        
	}
    
    
    /**
    * Cronjob Aufruf des Multithread Schedulers
    */
    public function scheduler()
    {
        $id=(int)gmdate("si");
        mage::log("Cronjob $id start, Call:scheduler mit ProzessID:".getmypid()." und SessionId:".session_id()." gelaufen um ".gmdate("Y-m-d H:i:s"),zend_log::INFO,self::$log_file);
        
        $obj= new Heliumv_Synchronization_Model_Scheduler();
        $obj->runScheduler();
        //$this->wait5minutesandwritestatus2log($id);
        //$this->wait5minutesandwritestatus2log(getmypid()."_2_started_".gmdate("Y-m-d H:i:s"));
        
        mage::log("Cronjob $id end, Call:scheduler",zend_log::INFO,self::$log_file);
    }
    
    /**
    * Cronjob aufruf um gesamten Kundenstamm aus HV zu übern ehmen mit Aufruf CustomerSync("full")
    */
    public function customerimport()
    {
        $id=(int)gmdate("si");
        mage::log("Cronjob $id start, Call:customerimport mit ProzessID:".getmypid()." und SessionId:".session_id()." gelaufen um ".gmdate("Y-m-d H:i:s"),zend_log::INFO,self::$log_file);
        $obj= new Heliumv_Synchronization_Model_Customer();
        $obj->doCustomerSync("full");
        mage::log("Cronjob $id end, Call:customerimport",zend_log::INFO,self::$log_file);
    }
    
    /**
    * Cronjob aufruf um Kundenstamm zu aktualisieren mit Aufruf CustomerSync("update")
    */
    public function customerupdate()
    {
        $id=(int)gmdate("si");
        mage::log("Cronjob $id start, Call:customerupdate mit ProzessID:".getmypid()." und SessionId:".session_id()." gelaufen um ".gmdate("Y-m-d H:i:s"),zend_log::INFO,self::$log_file);
        $obj= new Heliumv_Synchronization_Model_Customer();
        $obj->doCustomerSync("update");
        mage::log("Cronjob $id end, Call:customerupdate",zend_log::INFO,self::$log_file);
    }
    
    
    
    /**
    * Cronjob aufruf um alle Artikel und Categorien aus HV zu importieren mit Aufruf doArticleSync("full")
    */
    public function articleimport()
    {
        $id=(int)gmdate("si");
        mage::log("Cronjob $id start, Call:articleimport mit ProzessID:".getmypid()." und SessionId:".session_id()." gelaufen um ".gmdate("Y-m-d H:i:s"),zend_log::INFO,self::$log_file);
        $obj= new Heliumv_Synchronization_Model_Article();
        $obj->doArticleSync("full");
        mage::log("Cronjob $id end, Call:articleimport",zend_log::INFO,self::$log_file);
    }
    
    
    /**
    * Cronjob aufruf um Artikel und Kategorien zu aktualisieren mit Aufruf doArticleSync("update")
    */
    public function articleupdate()
    {
        $id=(int)gmdate("si");
        mage::log("Cronjob $id start, Call:articleupdate mit ProzessID:".getmypid()." und SessionId:".session_id()." gelaufen um ".gmdate("Y-m-d H:i:s"),zend_log::INFO,self::$log_file);
        $obj= new Heliumv_Synchronization_Model_Article();
        $obj->doArticleSync("update");
        mage::log("Cronjob $id end, Call:articleupdate",zend_log::INFO,self::$log_file);
    }
    
    /**
    * Cronjob aufruf um alle Offenen bestedllungen  nach HV zu übertragen
    */
    public function ordersync()
    {
        $id=(int)gmdate("si");
        mage::log("Cronjob $id start, Call:ordersync mit ProzessID:".getmypid()." und SessionId:".session_id()." gelaufen um ".gmdate("Y-m-d H:i:s"),zend_log::INFO,self::$log_file);
        $obj= new Heliumv_Synchronization_Model_Order();
        $obj->doOrderSync();
        mage::log("Cronjob $id end, Call:ordersync",zend_log::INFO,self::$log_file);
    }
    
    
    
    /**
    * Testaufruf für Cronjob
    * benötigt eintrag in config.xml und laufenden aufruf von localhost/magento2/cron.php
    * 
    * @param mixed $schedule
    */
    /*public function test($schedule)
    {
        $id=(int)gmdate("si");
        mage::log("Cronjob $id Call:test mit ProzessID:".getmypid()." und SessionId:".session_id()." gelaufen um ".gmdate("Y-m-d H:i:s"),zend_log::INFO,self::$log_file);
        
        //startet order sync verarbeitung testweise via cronjob
        $order_model= new Heliumv_Synchronization_Model_Order;
        //$order_model->doOrderSync();
        
        //$article_obj=new Heliumv_Synchronization_Model_Article();
        //$article_obj->doArticleSync("full");
        
        $this->wait5minutesandwritestatus2log($id);
        //$this->wait5minutesandwritestatus2log(getmypid()."_2_started_".gmdate("Y-m-d H:i:s"));
        
        mage::log("Cronjob $id end, Call:test",zend_log::INFO,self::$log_file);
        
    }*/
    
    
    /**
    * Test Function für multithread, wartet 5 minuten und schreibt alle 10 sekunden einen logeintrag
    */
    /*public function wait5minutesandwritestatus2log($thread_id="")
    {

        //$end=time()+5*60;
        $end=time()+2*60;
        mage::log("ThreadID:".$thread_id." Started",null,"multithread_tests.txt");    
        while(time()<$end)
        {
            sleep(10);
            mage::log("ThreadID:".$thread_id." alive",null,"multithread_tests.txt");    
        }
        mage::log("ThreadID:".$thread_id." Ende",null,"multithread_tests.txt");    
    } */
    

}